@@ -4,9 +4,8 @@ use strict;
 use warnings;
 
 require Carp;
-require Object::Simple::Error;
 
-our $VERSION = '0.01_02';
+our $VERSION = '0.0201';
 
 # meta imformation( accessor option of each class )
 our $META = {};
@@ -15,40 +14,34 @@ our $META = {};
 our @ATTRIBUTES_INFO;
 
 # valid import option value
-my %VALID_IMPORT_OPTIONS = map{ $_ => 1 } qw( base mixin );
+my %VALID_IMPORT_OPTIONS = map{$_ => 1} qw(base mixin);
 
 # import
-sub import{
-    
-    my ( $self, @opts ) = @_;
+sub import {
+    my ($self, %options) = @_;
     
     # shortcut
     return unless $self eq 'Object::Simple';
     
-    # arrange arguments
-    @opts = %{ $opts[0] } if ref $opts[0] eq 'HASH';
-    
     # check import option
-    my $import_options = {};
-    while( my ( $opt, $val ) = splice( @opts, 0, 2 ) ){
-        Carp::croak "Invalid import option '$opt'" unless $VALID_IMPORT_OPTIONS{ $opt };
-        $import_options->{ $opt } = $val;
+    foreach my $key (keys %options) {
+        Carp::croak "Invalid import option '$key'" unless $VALID_IMPORT_OPTIONS{ $key };
     }
     
     # get caller package name
     my $caller_class = caller;
     
     # inherit base class;
-    Object::Simple::Functions::inherit_base_class( $caller_class, $import_options->{ base } );
+    Object::Simple::Functions::inherit_base_class($caller_class, $options{base});
     
     # inherit Object::Simple;
     {
         no strict 'refs';
-        push @{ "${caller_class}::ISA" }, 'Object::Simple';
+        push @{"${caller_class}::ISA"}, 'Object::Simple';
     }
     
     # import methods form mixin class;
-    Object::Simple::Functions::import_methods_mixin_class( $caller_class, $import_options->{ mixin } );
+    Object::Simple::Functions::import_methods_mixin_class( $caller_class, $options{mixin});
     
     # auto strict and auto warnings
     strict->import;
@@ -61,7 +54,6 @@ sub import{
 
 # new
 sub new{
-    
     my $invocant = shift;
     
     # convert to class name
@@ -130,16 +122,13 @@ sub end{
         my $attr = $attr_names->{ $class }{ $ref };
         
         # get accessor option
-        my $attr_options = [ $ref->() ];
+        my $attr_options = { $ref->() };
         
         # check accessor option
-        Object::Simple::Functions::check_accessor_option( $attr, $class, @$attr_options );
-        
-        # accessor option convert to hash reference
-        $attr_options = { @$attr_options };
+        Object::Simple::Functions::check_accessor_option( $attr, $class, $attr_options );
         
         # resist accessor option to meta imformation
-        $Object::Simple::META->{ attr }{ $class }{ $attr } = $attr_options;
+        $Object::Simple::META->{ attr_options }{ $class }{ $attr } = $attr_options;
         
         # create accessor
         {
@@ -166,6 +155,8 @@ sub error{
     my $error = $@;
     
     my $is_object_simple_error = eval{ $error->isa( 'Object::Simple::Error' ) };
+
+    require Object::Simple::Error;
     my $error_object
         = $is_object_simple_error ? $error :
                                     Object::Simple::Error->new( message => "$error", position => '' );
@@ -175,15 +166,50 @@ sub error{
 }
 
 package Object::Simple::Functions;
-use strict;
-use warnings;
-
-use Object::Simple::Error;
-use Scalar::Util;
-use Storable;
-use Class::ISA;
 use Object::Simple::Constraint;
 
+# copied from Mouse
+BEGIN {
+    my $impl;
+    if ($] >= 5.009_005) {
+        require mro;
+        $impl = \&mro::get_linear_isa;
+    } else {
+        my $loaded = do {
+            local $SIG{__DIE__} = 'DEFAULT';
+            eval { require MRO::Compat; 1 };
+        };
+        if ($loaded) {
+            $impl = \&mro::get_linear_isa;
+        } else {
+#       VVVVV   CODE TAKEN FROM MRO::COMPAT   VVVVV
+            my $code; # this recurses so it isn't pretty
+            $code = sub {
+                no strict 'refs';
+
+                my $classname = shift;
+
+                my @lin = ($classname);
+                my %stored;
+                foreach my $parent (@{"$classname\::ISA"}) {
+                    my $plin = $code->($parent);
+                    foreach (@$plin) {
+                        next if exists $stored{$_};
+                        push(@lin, $_);
+                        $stored{$_} = 1;
+                    }
+                }
+                return \@lin;
+            };
+#       ^^^^^   CODE TAKEN FROM MRO::COMPAT   ^^^^^
+            $impl = $code;
+        }
+    }
+
+    no strict 'refs';
+    *{ __PACKAGE__ . '::get_linear_isa'} = $impl;
+}
+
 sub inherit_base_class{
     
     my ( $caller_class, $base ) = @_;
@@ -304,7 +330,7 @@ sub parse_mixin_class_method_info{
 }
 
 # create instance
-sub create_instance{
+sub create_instance {
     
     my ( $invocant, $args ) = @_;
     
@@ -314,14 +340,14 @@ sub create_instance{
     bless $self, $class;
     
     # merge self and parent accessor option
-    my $attr_options = merge_self_and_super_accessor_option( $class );
+    my $attr_options = merge_self_and_super_accessor_option($class);
     
     # initialize hash slot
-    foreach my $attr ( keys %{ $attr_options } ){
-        my $arg = delete $args->{ $attr };
-        my $required = $attr_options->{ $attr }{ required };
+    foreach my $attr (keys %{$attr_options}) {
+        my $required = $attr_options->{$attr}{required};
         
-        if( $required && !defined $arg ){
+        if($required && !exists $args->{$attr}) {
+            require Object::Simple::Error;
             Object::Simple::Error->throw(
                 type => 'attr_required',
                 message => "Attr '$attr' is required.",
@@ -330,34 +356,42 @@ sub create_instance{
             );            
         }
         
-        if( defined $arg ){
-            $self->$attr( $arg );
+        if(exists $args->{$attr}) {
+            $self->$attr($args->{$attr});
+            delete $args->{$attr}
         }
-        elsif( my $default = $attr_options->{ $attr }{ default } ){
-            
-            $self->{ $attr } = ref $default ? Storable::dclone( $default ) :
-                                              $default;
+        elsif(my $default = $attr_options->{$attr}{default} ){
+            if(ref $default) {
+                require Storable;
+                $self->{$attr} = Storable::dclone($default);
+            }
+            else {
+                $self->{$attr} = $default;
+            }
         }
     }
-    
     return $self;
-    
 }
 
 # marge self and super accessor option
-sub merge_self_and_super_accessor_option{
+sub merge_self_and_super_accessor_option {
     
     my $class = shift;
     
-    my @self_and_super_classes = reverse Class::ISA::self_and_super_path($class);
+    return $Object::Simple::META->{merged_attr_options}{$class}
+      if $Object::Simple::META->{merged_attr_options}{$class};
+    
+    my $self_and_super_classes
+      = Object::Simple::Functions::get_linear_isa($class);
     my $attr_options = {};
     
-    foreach my $class ( @self_and_super_classes ){
-        $attr_options = { %{ $attr_options }, %{ $Object::Simple::META->{ attr }{ $class } } }
-            if defined $Object::Simple::META->{ attr }{ $class }
+    foreach my $class (reverse @$self_and_super_classes){
+        $attr_options = {%{$attr_options}, %{$Object::Simple::META->{attr_options}{$class}}}
+            if defined $Object::Simple::META->{attr_options}{$class};
     }
-    return $attr_options;
     
+    $Object::Simple::META->{merged_attr_options}{$class} = $attr_options;
+    return $attr_options;
 }
 
 # type constraint functions
@@ -421,6 +455,7 @@ sub create_accessor{
     if ( $attr_options->{ read_only } ){
         $e .=
             qq/    if( \@_ ){\n/ .
+            qq/        require Object::Simple::Error;\n/ .
             qq/        Object::Simple::Error->throw(\n/ .
             qq/            type => 'read_only',\n/ .
             qq/            message => "${class}::$attr is read only",\n/ .
@@ -451,6 +486,7 @@ sub create_accessor{
             
             $e .=
             qq/        if( !\$ret ){\n/ .
+            qq/            require Object::Simple::Error;\n/ .
             qq/            Object::Simple::Error->throw(\n/ .
             qq/                type => 'type_invalid',\n/ .
             qq/                message => "${class}::$attr Type error",\n/ .
@@ -477,6 +513,7 @@ sub create_accessor{
             qq/        \$self->{ $attr } = \$_[0];\n\n/;
         
         if( $attr_options->{ weak } ){
+            require Scalar::Util;
             $e .=
             qq/        Scalar::Util::weaken( \$self->{ \$attr } )\n/ .
             qq/            if ref \$self->{ $attr };\n\n/;
@@ -505,7 +542,6 @@ sub create_accessor{
             qq/}\n/;
     
     return $e;
-    
 }
 
 # accessor option
@@ -516,36 +552,31 @@ my %VALID_ATTR_OPTIOTNS
 sub check_accessor_option{
     
     # accessor info
-    my ( $attr, $class, @attr_options ) = @_;
+    my ( $attr, $class, $attr_options ) = @_;
     
     my $hook_options_exist = {};
-    while( my( $key, $val ) = splice( @attr_options, 0, 2 ) ){
+    foreach my $key ( keys %$attr_options ){
         Carp::croak "${class}::$attr '$key' is invalid accessor option" 
             unless $VALID_ATTR_OPTIOTNS{ $key };
     }
-    
 }
 
 sub define_MODIFY_CODE_ATTRIBUTES{
+    my $class = shift;
     
-    my $caller_class = shift;
-    my $e .=
-        qq/package ${caller_class};\n/ .
-        qq/sub MODIFY_CODE_ATTRIBUTES {\n/ .
-        qq/\n/ .
-        qq/    my (\$class, \$ref, \@attrs) = \@_;\n/ .
-        qq/    if( \$attrs[0] eq 'Attr' ){\n/ .
-        qq/        push( \@Object::Simple::ATTRIBUTES_INFO, [\$class, \$ref ]);\n/ .
-        qq/    }\n/ .
-        qq/    else{\n/ .
-        qq/        die "'\$attrs[0]' is bad. attribute must be 'Attr'";\n/ .
-        qq/    }\n/ .
-        qq/    return;\n/ .
-        qq/}\n/;
-    
-    eval $e;
-    if( $@ ){ die "Cannot execute\n $e" }; # never occured.
+    my $code = sub {
+        my ($class, $ref, @attrs) = @_;
+        if( $attrs[0] eq 'Attr' ){
+            push( @Object::Simple::ATTRIBUTES_INFO, [$class, $ref ]);
+        }
+        else{
+            die "'$attrs[0]' is bad. attribute must be 'Attr'";
+        }
+        return;
+    };
     
+    no strict 'refs';
+    *{"${class}::MODIFY_CODE_ATTRIBUTES"} = $code;
 }
 
 =head1 NAME
@@ -554,7 +585,7 @@ Object::Simple - Very simple framework for Object Oriented Perl.
 
 =head1 VERSION
 
-Version 0.01_02
+Version 0.0201
 
 =cut
 
@@ -580,8 +611,6 @@ The feature is that
 
 =item 3. You can define default value of attribute.
 
-=item 4. Error object is thrown, when error is occured.
-
 =back
 
 If you use Object::Simple, you are free from bitter work 
@@ -640,8 +669,12 @@ writing new methods and accessors repeatedly.
     
     # Mixin
     package Book;
-    use Object::Simple( mixin => [ 'Object::Simple::Mixin::Meta' ] );
-    
+    use Object::Simple( 
+        mixin => [ 
+            'Object::Simple::Mixin::AttrNames',
+            'Object::Simple::Mixin::AttrOptions'
+        ]
+    );
 
 =cut
 
@@ -656,13 +689,136 @@ new method is prepared.
 
 =head2 _arrange_args
 
-You can override this method to arrange arguments.
+This method receive hash or hash reference, and return hash ref by default.
+
+You can override this method to arrange arguments like this.
+
+    sub _arrange_args {
+        my ($self, $x, $y ) = @_;
+        return {x => $x, y => $y};
+    }
+
+This method must retrun hash ref.
+
+=head2 _init
+
+You can initialize object.
+
+You can override this method
+
+This method receive hash ref argments except attribrte by defined by Attr
+
+    sub _init {
+        my ($self, $args) = @_;
+        
+    }
+    
+=head2 error
+
+You can get error as Object::Simple::Error object
+
+    my $error = Object::Simple->error;
 
 =head2 end
 
 resist attribute and create accessors.
 
-    Object::Simple->end
+Script must end 'Object::Simple->end;'
+
+    Object::Simple->end;
+
+
+=head1 ACCESSOR OPTIONS
+
+=head2 default
+
+You can define attribute default value
+
+    sub title : Attr {default => 'Good news'}
+    sub author : Attr {default => ['Ken', 'Taro']}
+
+=head2 auto_build
+
+You can automaticaly set attribute value when accessor is called first.
+    
+    sub author : Attr { auto_build => 1 }
+    sub build_author{
+        my $self = shift;
+        $self->atuhor( Person->new );
+   }
+
+Builder method name is build_ATTRIBUTE_NAME by default;
+
+You can specify build method .
+
+    sub author : Attr { auto_build => 1 }
+    sub create_author{
+        my $self = shift;
+        $self->atuhor( Person->new );
+    }
+
+=head2 type
+
+You can define type of attribute value
+    
+    sub price: Attr {type => 'Int'} # must be integer
+    sub author: Attr {type => 'Person'} # must be inherit Perlson class
+
+list of default types
+
+    Bool       => \&Object::Simple::Constraint::is_bool,
+    Undef      => sub { !defined($_[0]) },
+    Defined    => sub { defined($_[0]) },
+    Value      => \&Object::Simple::Constraint::is_value,
+    Num        => \&Object::Simple::Constraint::is_num,
+    Int        => \&Object::Simple::Constraint::is_int,
+    Str        => \&Object::Simple::Constraint::is_str,
+    ClassName  => \&Object::Simple::Constraint::is_class_name,
+    Ref        => sub { ref($_[0]) },
+
+    ScalarRef  => \&Object::Simple::Constraint::is_scalar_ref,
+    ArrayRef   => \&Object::Simple::Constraint::is_array_ref,
+    HashRef    => \&Object::Simple::Constraint::is_hash_ref,
+    CodeRef    => \&Object::Simple::Constraint::is_code_ref,
+    RegexpRef  => \&Object::Simple::Constraint::is_regexp_ref,
+    GlobRef    => \&Object::Simple::Constraint::is_glob_ref,
+    FileHandle => \&Object::Simple::Constraint::is_file_handle,
+    Object     => \&Object::Simple::Constraint::is_object
+
+You can specify code reference
+
+    sub price: Attr {type => sub{ $_[0] =~ /^\d+$/ }}
+
+=head2 read_only
+
+You can create read only accessor
+    
+    sub title: Attr { read_only => 1 }
+
+=head2 setter_return
+
+You can spesicy return value when setting value
+
+    sub title : Attr { setter_return => 'old' }
+
+list of setter_return option
+
+    old
+    current
+    self
+    undef
+
+=head1 required
+
+You can specify required attribute when instance is created.
+
+    sub title : Attr {required => 1}
+
+=head1 weak
+
+attribute value is weak reference.
+
+    sub parent : Attr {weak => 1}
 
 =head1 SEE ALSO
 
@@ -686,7 +842,6 @@ You can find documentation for this module with the perldoc command.
 
     perldoc Object::Simple
 
-
 You can also look for information at:
 
 =over 4
